import 'package:async_injector/error/dependency_is_null.dart';
import 'package:async_injector/provide/module.dart';
import 'package:async_injector/scope/scope.dart';
import 'provider.dart';

class Component extends Provider {
  Component(Scope scope) : super(scope);

  addDependency<T>(T value) {
    scope.add<T>(value);
  }
}

class ComponentBuilder {
  final scope = Scope();
  List<Module> initQueue = [];

  module(Module module) {
    initQueue.add(module);
  }
  moduleImpl<T>(Function(Scope) onCreate) {
    initQueue.add(ModuleImpl<T>(onCreate));
  }
  dependency<T>(T value) {
    scope.add<T>(value);
  }

  //remove after build
  buildDependency<T>(T value) {
    scope.addBuildDependencies<T>(value);
  }

  Future<Component> build() async {
    await initDependencies();
    return Component(scope);
  }

  initDependencies() async {
    for (Module module in initQueue) {
      module.scope = scope;
      var moduleInit = module.init();

      var dependency =
          moduleInit is Future<dynamic> ? await moduleInit : moduleInit;

      if (dependency == null) {
        throw DependencyIsNull(module);
      }
      scope.add(dependency, type: module.type);
    }
    scope.buildDependencies.clear();
    initQueue.clear();
  }
}
